@@ -1,7 +1,21 @@
-//@line 38 "/media/disc0/rl/ports/x/firefox/work/src/mozilla/toolkit/mozapps/downloads/content/helperApps.js"
+//@line 38 "/media/disc0/rl/ports/x/firefox3/work/src/mozilla/toolkit/mozapps/downloads/content/helperApps.js"
+
+var Cc = Components.classes;
+var Ci = Components.interfaces;
 
 var gRDF;    
 
+const CLASS_MIMEINFO        = "mimetype";
+const CLASS_PROTOCOLINFO    = "scheme";
+
+// namespace prefix
+const NC_NS                 = "http://home.netscape.com/NC-rdf#";
+
+// type list properties
+
+const NC_MIME_TYPES         = NC_NS + "MIME-types";
+const NC_PROTOCOL_SCHEMES   = NC_NS + "Protocol-Schemes";
+
 ///////////////////////////////////////////////////////////////////////////////
 // MIME Types DataSource Wrapper
 
@@ -25,34 +39,33 @@ function APP_URI(aType)
   return "urn:mimetype:externalApplication:" + aType;
 }
 
-function ArrayEnumerator(aItems)
-{
-  this._index = 0;
-  
+function ArrayEnumerator(aItems) {
   if (aItems) {
-    for (var i = 0; i < aItems.length; ++i) {    
+    for (var i = 0; i < aItems.length; ++i) {
       if (!aItems[i])
-        aItems.splice(i, 1);      
+        aItems.splice(i--, 1);
     }
+    this._contents = aItems;
+  } else {
+    this._contents = [];
   }
-  
-  this._contents = aItems || [];
+}
 
-  this.push = function (aElement) 
-  {
+ArrayEnumerator.prototype = {
+  _index: 0,
+
+  hasMoreElements: function () {
+    return this._index < this._contents.length;
+  },
+
+  getNext: function () {
+    return this._contents[this._index++];
+  },
+
+  push: function (aElement) {
     if (aElement)
       this._contents.push(aElement);
-  };
-  
-  this.hasMoreElements = function ()
-  {
-    return this._index < this._contents.length;
-  };
-  
-  this.getNext = function ()
-  {
-    return this._contents[this._index++];      
-  };
+  }
 };
 
 function HelperApps()
@@ -126,12 +139,12 @@ HelperApps.prototype = {
         entry.appPath = "";
       }
     } 
-    else {
+    else if (aMIMEInfo.preferredApplicationHandler instanceof Components.interfaces.nsILocalHandlerApp) {
       entry.saveToDisk       = false;
       entry.useSystemDefault = false;
       entry.handleInternal   = false;
-      entry.appPath = aMIMEInfo.preferredApplicationHandler.path;
-      entry.appDisplayName = aMIMEInfo.applicationDescription;
+      entry.appPath = aMIMEInfo.preferredApplicationHandler.executable.path;
+      entry.appDisplayName = aMIMEInfo.preferredApplicationHandler.name;
     }
     
     // Do RDF magic.
@@ -174,11 +187,8 @@ HelperApps.prototype = {
     if (types) {
       types = types.QueryInterface(Components.interfaces.nsIRDFLiteral);
       types = types.Value.split(", ");
-      
-      // We're using helper app service as our MIME Service here because the helper app service
-      // talks to OS Specific hooks that on some platforms (MacOS X) are required to get a 
-      // fully populated MIME Info object. Thus it is this object that we return. 
-      mimeSvc = Components.classes["@mozilla.org/uriloader/external-helper-app-service;1"].getService(Components.interfaces.nsIMIMEService);
+
+      mimeSvc = Components.classes["@mozilla.org/mime;1"].getService(Components.interfaces.nsIMIMEService);
       return mimeSvc.getFromTypeAndExtension(types[0], null);
     }
     
@@ -307,12 +317,6 @@ HelperApps.prototype = {
     }
     return this._inner.HasAssertion(aSource, aProperty, aTarget, aTruthValue);
   },
-  AddObserver: function (aObserver) {
-    this._inner.AddObserver(aObserver);
-  },
-  RemoveObserver: function (aObserver) {
-    this._inner.RemoveObserver(aObserver);
-  },
   ArcLabelsIn: function (aNode) {
     return this._inner.ArcLabelsIn(aNode);
   },
@@ -367,6 +371,18 @@ HelperApps.prototype = {
     }
   },
   
+  onBeginUpdateBatch: function (aDataSource) {
+    for (var i = 0; i < this._observers.length; ++i) {
+      this._observers[i].onBeginUpdateBatch(aDataSource);
+    }
+  },
+  
+  onEndUpdateBatch: function (aDataSource) {
+    for (var i = 0; i < this._observers.length; ++i) {
+      this._observers[i].onEndUpdateBatch(aDataSource);
+    }
+  },
+  
   beginUpdateBatch: function (aDataSource) {
     for (var i = 0; i < this._observers.length; ++i) {
       this._observers[i].beginUpdateBatch(aDataSource);
@@ -578,17 +594,10 @@ HandlerOverride.prototype = {
     var helperAppResource = gRDF.GetUnicodeResource(APP_URI(this.mimeType));
     this._DS.Assert(handlerResource, helperAppProperty, helperAppResource, true);
     // add the mime type to the MIME types seq
-    var container = Components.classes["@mozilla.org/rdf/container;1"].createInstance();
-    if (container) {
-      container = container.QueryInterface(Components.interfaces.nsIRDFContainer);
-      if (container) {
-        var containerRes = gRDF.GetUnicodeResource("urn:mimetypes:root");
-        container.Init(this._DS, containerRes);
-        var element = gRDF.GetUnicodeResource(MIME_URI(this.mimeType));
-        if (container.IndexOf(element) == -1)
-          container.AppendElement(element);
-      }
-    }
+    var container = this.ensureAndGetTypeList("mimetype");
+    var element = gRDF.GetUnicodeResource(MIME_URI(this.mimeType));
+    if (container.IndexOf(element) == -1)
+      container.AppendElement(element);
   }, 
   
   // Implementation helper methods
@@ -660,6 +669,43 @@ HandlerOverride.prototype = {
     var valueProperty = gRDF.GetUnicodeResource(NC_URI(aPropertyString));
     var mimeLiteral = gRDF.GetLiteral(aValueString);
     this._DS.Unassert(mimeSource, valueProperty, mimeLiteral, true);
+  },
+
+  /**
+   * Get the list of types for the given class, creating the list if it doesn't
+   * already exist. The class can be either CLASS_MIMEINFO or CLASS_PROTOCOLINFO
+   * (i.e. the result of a call to _getClass).
+   * 
+   * |urn:<class>s|
+   * |urn:<class>s:root|
+   * 
+   * @param aClass {string} the class for which to retrieve a list of types
+   *
+   * @returns {nsIRDFContainer} the list of types
+   */
+  ensureAndGetTypeList: function (aClass) {
+    var source = gRDF.GetResource("urn:" + aClass + "s");
+    var property =
+      gRDF.GetResource(aClass == CLASS_MIMEINFO ? NC_MIME_TYPES
+                                                : NC_PROTOCOL_SCHEMES);
+    var target = gRDF.GetResource("urn:" + aClass + "s:root");
+
+    // Make sure we have an arc from the source to the target.
+    if (!this._DS.HasAssertion(source, property, target, true))
+      this._DS.Assert(source, property, target, true);
+
+    // Make sure the target is a container.
+    var containerUtils = Cc["@mozilla.org/rdf/container-utils;1"]
+                            .getService(Ci.nsIRDFContainerUtils);
+    if (!containerUtils.IsContainer(this._DS, target))
+      containerUtils.MakeSeq(this._DS, target);
+
+    // Get the type list as an RDF container.
+    var typeList =
+          Cc["@mozilla.org/rdf/container;1"].createInstance(Ci.nsIRDFContainer);
+    typeList.Init(this._DS, target);
+
+    return typeList;
   }
 };
 
